from rest_framework import serializers

from .models import TournamentModel

from profiles.models import ProfileModel
from profiles.serializers.ProfileSerializer import ProfileSerializer
from games.serializers import GameSerializer

nb_participants = [2 ** i for i in range(2, 6)]

class TournamentSerializer(serializers.ModelSerializer):

    state = serializers.SerializerMethodField(read_only=True, required=False)
    participants = serializers.SerializerMethodField(read_only=True, required=False)
    round = serializers.ReadOnlyField()
    started = serializers.ReadOnlyField()
    finished = serializers.ReadOnlyField()
    name = serializers.CharField(default="")

    class Meta:
        model = TournamentModel
        fields = ["name", "nb_participants", "round", "started", "finished", "id", "state", "participants"]
    
    def get_participants(self, instance: TournamentModel):
        return ProfileSerializer(instance.get_participants(), many=True).data

    def get_state(self, instance: TournamentModel):
        return ["waiting", "started", "finished"][instance.started + instance.finished]

    def validate_nb_participants(self, value: int):
        if (value not in nb_participants):
            raise serializers.ValidationError(f"The numbers of participants must be {str(nb_participants)}.")
        return value